/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ACCOUNTJOBXMLWRITER_H
#define ACCOUNTJOBXMLWRITER_H

#include <list>
#include "hobit.h"
#include "hobit_validator.h"
#include "hobit_creator.h"
#include "aqmoney2/aqmapi.h"
#include "aqmoney2/accountjobs.h"
#include "aqmoney2/account.h"
#include <openhbci2.h>

#define HOBIT_ENCODING "UTF-8"


// predeclaration of class hobit
class HoBIT;
using namespace HBCI;

/**
Writes created account jobs to an xml-file. AccountJobFactory uses this class to store generated account jobs after their creation.

@author Hendrik Sirges
*/
class AccountJobXMLWriter{
public:
    AccountJobXMLWriter(HoBIT* hobit, AqMoneyAPI* api, const char *dirpath);

    ~AccountJobXMLWriter();
    HobitError writeJobGetDatedTransfers(	int country, 
											const string &bankId, 
											const string &accountId, 
											Date fromdate, 
											Date todate);
											
    HobitError writeJobGetBalance();
	
    HobitError writeJobGetStandingOrders(	int country, 
											const string &bankId, 
											const string &accoundId);
											
    HobitError writeJobGetTransactions(	Date fdate, 
										Date tdate, 
										int country, 
										const string &bankId, 
										const string &accoundId);
										
    HobitError writeJobSingleTransfer(	bool skipBad,
										int country,
										const string &bankCode,
										const string &accountId,
										const string &accountSubId,
										int otherCountry,
										const string& otherBankCode,
										const string& otherAccount,
										const list<string> &otherName,
										const list<string> &purpose,
										const  Value&  value,
										int textkey,
										const Date& date);
										
    HobitError writeJobDeleteDatedTransfer(	bool skipBad,
											int country,
											const string &bankCode,
											const string &accountId,
											const string &accountSubId,
											int otherCountry,
											const string& otherBankCode,
											const string& otherAccount,
											const list<string> &otherName,
											const list<string> &purpose,
											const  Value&  value,
											int textkey,
											int textkeyExt,
											const Date& date,
											const string& jobId);

	HobitError writeJobSingleDebitNote(	bool skipBad,
										int country,
										const string &bankCode,
										const string &accountId,
										const string &accountSubId,
										int otherCountry,
										const string& otherBankCode,
										const string& otherAccount,
										const list<string> &otherName,
										const list<string> &purpose,
										const  Value&  value,
										int textkey,
										const Date& date);
										
    HobitError writeJobStartStandingOrder(	bool skipBad,
											int country,
											const string &bankCode,
											const string &accountId,
											const string &accountSubId,
											int otherCountry,
											const string& otherBankCode,
											const string& otherAccount,
											const list<string> &otherName,
											const list<string> &purpose,
											const  Value&  value,
											int textkey,
											const Date& date,
											const Date& firstdate,
											const Date& lastdate,
											const int turnus,
											bool weekly,
											int day);
											
    HobitError writeJobDeleteStandingOrder(	bool skipBad,
											int country,
											const string& bankCode,
											const string& accountId,
											int otherCountry,
											const string& otherBankCode,
											const string& otherAccountId,
											const list<string>& otherName,
											const list<string>& purpose,
											const  Value&  value,
											int textkey,
											int textkeyExt,
											const Date& date,
											const Date& firstdate,
											const Date& lastdate,
											const string& jobId,
											int turnus,
											int day,
											bool weekly);
    
    private:
	// Hobits Pre-Queue for storing jobs
	Pointer<AqMoneyAPI> _api;
	Pointer<HoBIT>  _hobit;
	const char *_directorypath;
	const char *_uri;
	

};

#endif
