/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guidataexport.h"
#include "globals.h"
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <fstream>
using namespace std;

GuiDataExport::GuiDataExport(QWidget *parent, const char *name)
 : GuiDataExportBase(parent, name)
{
 _Encoding = QTextStream::Unicode;
 qDebug("Constructor GuiDataExport");
}


GuiDataExport::~GuiDataExport()
{
 qDebug("Destructor GuiDataExport");
}


void GuiDataExport::slotSelectFile(){
  QString keyFiles = tr("CSV Export Files (*.csv)");
  QString path = QFileDialog::getSaveFileName(_mountpoint.c_str(),keyFiles,this);
  
  if (path){
    _filepath = path.ascii();
    lineEditFile->setText(_filepath.c_str());
  }
}


void GuiDataExport::slotExport(){

   if (QMessageBox::question(this,
            tr("Really export?"),
            tr("Exporting data to a plain text format is a security risk. Are you sure?"),
            tr("&Yes"), tr("&No"),
            QString::null, 0, 1 ) )
        return;
   
   QString toExport = "";
   QFile file(lineEditFile->text().ascii());
   
   if (!file.open(IO_WriteOnly)) {
     QMessageBox::critical(this,tr("Failed"), 
     tr("Can't open export file"));
   }
   else{
     for (int row = 0; row < _table->numRows(); row++){    
       for (int col = 0; col < _table->numCols(); col++){

	// check if cell is a label (needed for colored cells)
	
	//&& _table->cellWidget(row, col)->name() == "QLabel")

	if (_table->cellWidget(row, col)) {
           QLabel* label = (QLabel*) _table->cellWidget(row, col);
           toExport += label->text();
	}
	else {
  	  toExport += _table->text(row,col);
	}
         toExport += ";";
       }//end for col
       toExport += "\n";
     }//end for row     
     QTextStream s(&file);
     s.setEncoding(_Encoding);
     s << toExport;
     QMessageBox::information(this,tr("Finished"),
     tr("The data have been exported to: ") + _filepath.c_str());
   }//end else
   
   file.close();
}

 
void GuiDataExport::slotBack(){
  emit back();
}


void GuiDataExport::setTableToExport(QTable* table){
  _table = table;
}


void GuiDataExport::setMountPoint(string mp){
	_mountpoint = mp;
	_filepath = _mountpoint + "/" + MONEYPENNY_DEFAULT_EXPORT_FILE;

	lineEditFile->setText(_filepath);
}
