/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qwidgetstack.h>
#include <qstring.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qcursor.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qregexp.h>

#include "guilog.h"
#include "globals.h"

GuiLog::GuiLog(QWidget* parent, const char* name)
    : GuiLogBase(parent,name)
{
     widgetStack->raiseWidget(1);
     _Jobs = 0;
     _JobsOk = 0;
     _JobsFailed = 0;
 //    setGeometry(0,0, width(), height());
     textEditLog->setTextFormat(Qt::RichText);
   qDebug("Constructor GuiLog");
}

GuiLog::~GuiLog()
{
   qDebug("Destructor GuiLog");
}


void GuiLog::setCursor ( const QCursor& cursor ){
GuiLogBase::setCursor(cursor);
textEditLog->setCursor(cursor);
}

/** Show the log messages.
*/
void GuiLog::slotShowDetails(){
 widgetStack->raiseWidget(0);
}
/** Hide the log messages.
*/
void GuiLog::slotHideDetails(){
 widgetStack->raiseWidget(1);
}

/** Set the text on job number label.
*/
void GuiLog::setJobText(const QString & jobText){
 labelPresentJob->setText(jobText);
}

/** Adds the text to editLog, and increments job number on label present job.
*/
void GuiLog::addLogText(QString & logText){
  QRegExp errorCodeExp ("9[0-9][0-9][0-9]"); //look for a 9000 code. 9000 to 9999 are error codes.
  if(logText.find(" job", 0, false)== 0) //from position 0, not case sensitive 
    _Jobs ++;
  if(_Jobs == 0)
    labelPresentJob->setText(tr("Starting job execution..."));
  else labelPresentJob->setText(tr("Executing job queue. Please wait."));
  if(logText.find("OK", 0, true)>= 0) //from position 0, case sensitive 
    _JobsOk ++;
  //search for error messages or error codes  
  if((logText.find  
   (tr("failed", "THIS must BE THE SAME WORD AS USED IN Job messages, which are generated by the bank server!!!" ), 0, false)>= 0) 
   | (errorCodeExp.search(logText) > 0)){ //from position 0, not case sensitive 
    _JobsFailed ++;
    logText = "<p> <b>" + logText + "</b></p>";
  } 
  else logText = "<p>" + logText + "</p>";
  textEditLog->setText(textEditLog->text() + "\n" + logText);
  qApp->processEvents();
}

/** Sets the text in edit, and increments job number on label present job.
*/
void GuiLog::setLogText(QString & logText){
  textEditLog->setText(logText);
  _Jobs = 0;
  _JobsOk = 0;
  _JobsFailed = 0;
  labelPresentJob->setText(tr("Preparing to execute jobs."));
}
  /** called on closeButton clicked
  */
  void GuiLog::slotCloseRequest(){
     emit closeRequest();
  }

/** Shows the results of the execution
*/
void GuiLog::showResults(){
 QString msg = 
   tr("<h3>Results of Job Execution</h3>")+
   "<ul>" +
     "<li>" + tr("Number of Jobs executed: ") + QString::number(_Jobs) +
     "<li>" + tr("Number of Jobs OK: ") + QString::number(_JobsOk) +
     "<li>" + tr("Number of failed Jobs: ") + QString::number(_JobsFailed) +
   "</ul>";
   if(_JobsFailed != 0)
    msg += tr("For details please press Show Details button. The failed jobs are printed bold.");
   labelPresentJob->setText(msg);
}
