/***************************************************************************
                          guiconnections.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guitools.h"
#include "globals.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qvalidator.h>
#include <qfiledialog.h>

#include <qgroupbox.h>

/**
 *this is the constructor for create new user
 *
 *@param main
 *@param parent the parent dialog of this widget
 *@param name widgets title
 *@param hobit
 */
GuiTools::GuiTools(QWidget *parent,GuiMain * guimain, const char *name, HoBIT* hobit)
    :GuiToolsBase(parent, name)
{
  _main = guimain;
  _hobit = hobit;
  _guicreateuser = new GuiCreateUser(this, _main, "Benutzer erstellen", _hobit);
  _guichangeversion = new GuiHbciVersion(this, _main, "Hbci Version", _hobit);
  updateList();

  // set order of tabs
  setTabOrder(listViewConnections, buttonCreateUser);
  setTabOrder(buttonCreateUser, buttonGetKeys);
  setTabOrder(buttonGetKeys, buttonSendKeys);
  setTabOrder(buttonSendKeys, buttonChangeHBCI);
  setTabOrder(buttonChangeHBCI, buttonGetSysId);
  setTabOrder(buttonGetSysId, buttonIniLetter);
  setTabOrder(buttonIniLetter, buttonGetAccounts);

  //hide the table header
  listViewConnections->header()->hide();
  QValidator* validator = new QIntValidator( 1, 65355, this );
  lineEditPort->setValidator(validator);

  // hide sql functions if not needed
  if (! USE_MYSQL_FEATURES) groupBoxDataAnalysis->hide();
  qDebug("Constructor GuiTools");

}

GuiTools::~GuiTools()
{
  qDebug("Destructor GuiTools");
}

/**
* starts dialog for creating a new user
*/
void GuiTools::slotCreateUser()
{
  //delete _guicreateuser;
  //_guicreateuser = new GuiCreateUser(_main, this, "Benutzer erstellen", _hobit);
  //_guicreateuser->show();
  //_main->refresh();
  _main->slotRaiseWidget(HBCI_WIZARD);
}


/**
* updates the list of users
*/
void GuiTools::updateList()
{

  listViewConnections->clear();
  Pointer<User>  user;
  list<Pointer<User> > users = _hobit->listUsers();
  Pointer<Bank> bank;
  Pointer<Medium> medium;


  if (!users.empty())
  {
    user = (*users.begin());

    bank =(user).ref().bank();
    medium =(user).ref().medium();
    QString bankname = bank.ref().bankCode().c_str();
    QString version, versions, num;

    // get used version
    num.setNum(bank.ref().hbciVersion());
    version = num;

    // get accepted versions
    list <int> vlist = bank.ref().hbciVersions();
    list <int>::iterator vit;
    for (vit = vlist.begin(); vit!= vlist.end(); vit++)
    {
      num.setNum(*vit);
      versions+= num;
      versions+= "      ";
    }


    new QListViewItem(listViewConnections, tr("Bank:\t"), bank.ref().name().c_str());
    new QListViewItem(listViewConnections, tr("Bankcode:\t"), bank.ref().bankCode().c_str());
    new QListViewItem(listViewConnections, tr("UserId:\t"), user.ref().userId().c_str());
    new QListViewItem(listViewConnections, tr("Security medium:\t"), medium.ref().mediumTypeName().c_str());
    new QListViewItem(listViewConnections, tr("SystemId:\t"), user.ref().systemId().c_str());
    new QListViewItem(listViewConnections, tr("Medium path:\t"), medium.ref().mediumName().c_str());
    new QListViewItem(listViewConnections, tr("Used HBCI-Version:\t"), version);
    new QListViewItem(listViewConnections, tr("Supported HBCI-Versions:\t"), versions);

  } // end if

  else
    new QListViewItem(listViewConnections, "State    ", "No HBCI-Configuration found");

}
/**
* Opens a Dialog to change the used HBCI version
*/
void GuiTools::slotChangeHbci()
{
  _guichangeversion->show();
  _main->refresh();
}

/**
* slot which retrieves the bank servers public keys 
*/
void GuiTools::slotGetKeys()
{
  _hobit->createJobGetKeys();
  if(_main->askForSend())
   _main->slotSendNow();
}

/**
* retrieves the system id of the bank server 
*/
void GuiTools::slotGetSystemId()
{
  _hobit->createJobGetSystemId();
  if(_main->askForSend())
   _main->slotSendNow();
}


/**
* sends our own public keys to the bank server 
*/
void GuiTools::slotSendKeys()
{
  _hobit->createJobSendKeys();
   if(_main->askForSend())
   _main->slotSendNow();
}

/**
* retrieves a list of our accounts 
*/
void GuiTools::slotGetAccounts()
{
  _hobit->createJobGetAccounts();
  if(_main->askForSend())
   _main->slotSendNow();
}


/**
* generates the customers iniletter 
*/
void GuiTools::slotGenerateIniLetter()
{
  _main->slotShowIniLetter();
}

/** Run a test connection to www.google.de or another url
*/
void GuiTools::slotConnectionTest(){
 
 HoBIT_Connector * connector = _main->getConnector();
 QString oldUrl = connector->getUrl();
 int oldPort = connector ->getPort();
 QString url = lineEditUrl->text();
 int port = lineEditPort->text().toInt();
 
 ConnectionState state = connector->connectionState();
 if(state == ConnectionConnected) 
   connector->closeConnection();  
 
 connector->setUrl("www.google.de");
 connector->setPort(80);
 connector->connectTCP();
 state = connector->connectionState();
 if(state == ConnectionConnected){
   connector->closeConnection();  
   QMessageBox::information(this, tr("Internet Connection Test"), 
     tr("Your internet setup seems to be Ok. Could connect to: ") + url + tr(" : Port: " + QString::number(port)));
 }
 else QMessageBox::warning(this, tr("Internet Connection Test"), 
       tr("Your internet setup does not work properly. Connection test failed. <br>") +
       tr("Tried to connect to: ") + url + tr(" : Port: " + QString::number(port) + tr("")));
 
 connector->setUrl(oldUrl);
 connector->setPort(oldPort);      
}
/** Start the hardware setup dialog
*/
void GuiTools
::slotHardwareSetup(){
  _process = new QProcess(_main);
  //_process->addArgument("kdesu");
  _process->addArgument("xterm");
  _process->start();
}

/** Emits signal help request.
*/
void GuiTools::slotHelp(){
 emit helpRequest("toolbox.html");
}
/** Emits signal importKey().
*/
void GuiTools::slotImportRDH2Key(){
 QString fileName = QFileDialog::getOpenFileName(                     
                    _main->getConfigDataDir(),
                    "RDH2 Keyfiles (*.key)",
                    this,
                    "Import RDH2 Keyfile",
                    "Choose a keyfile to import" );
 if(!fileName.isEmpty())
   emit importRDH2Key(fileName);
}

/** Emits signal sqlquery().
*/
void GuiTools::slotSQLQuery(){
emit sqlquery();
}
