/***************************************************************************
                          hobit.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "hobit.h"
#include "keyfilerdh2.h"
#include "keyfileopenhbci.h"

#include <qcursor.h>
#include <qfile.h>
#include <qcstring.h>
#include <qvalidator.h>
#include <qinputdialog.h>

#include <openhbci2/hbci.h>
#include <openhbci2/auth.h>
using namespace HBCI;


/* Default path for bank data */
#define BANKDATA_PATH "/usr/local/share/ktoblzcheck"

HoBIT::HoBIT(bool readonly,
             QWidget *parent,
             const string &openhbcifile,
             const string &aqmoneyfile,
             const string &templatefile,
             const string &hbcidatadir,
             const string &applicationname,
             HoBIT_Crypt* crypt,
             HoBIT_Connector * connector)
{     
     _readonly = readonly;
     _parent = parent;  
     _openhbcifile = openhbcifile; 
     _aqmoneyfile = aqmoneyfile; 
     _templatefile = templatefile;
     _hbcidatadir =  hbcidatadir;
     _applicationname = applicationname;
     _Crypt = crypt;
     _Connector = connector;
     
     cerr << "HbciDataDir is: " << _hbcidatadir << "\n";
     cerr << "OpenHbciFile is: " << _openhbcifile << "\n";
     cerr << "AqMoneyFile is: " << _aqmoneyfile << "\n";
     cerr << "TemplateFile is: " << _templatefile << "\n";
      
     createApi();
     qDebug("Constructor HoBIT"); 
}


HoBIT::~HoBIT(){
  //removeConfigDataDir();
  delete _AdminJobFactory;
  delete _AccountJobFactory;
  delete _Database;
  qDebug("Destructor HoBIT finished");
}


/**
*  executes both job queues for account and admin
*   jobs.
*/
HobitError HoBIT::executeJobQueue(){
      HobitError err1, err2;
      Pointer<Outbox> queue;
      Error err;
      bool errorFlag = false;
      HobitError result = HobitErrorSuccess;      
      Pointer<Medium> medium;
      list<Pointer<User> > users = listUsers();
      Pointer<User> user = (*users.begin());
      
      if(user.isValid()){ //adjust the _SecirityMode of the interactor , if possible
        medium = user.ref().medium();
        if(medium.isValid())
        _interactor.ref().setSecurityMode(medium.ref().securityMode());
      }
      //check for a working internet connection
      QString url = getBankServer( MONEYPENNY_DEFAULT_COUNTRY,"*" );
      _Connector->setUrl(url);
      qDebug("\nServer: %s", url.latin1());
      _Connector->connectTCP();
      ConnectionState state = _Connector->connectionState();
      
      
      if(state == ConnectionConnected){
        
	// NOW start execution of job queue

      try {
       DBG_INFO(0, "Creating new outbox");
       queue=new Outbox();

       _AdminJobFactory->updateJobQueue(queue.ptr());
       _AccountJobFactory->updateJobQueue(queue.ptr());  
       

        DBG_INFO(0, "Executing queue");
        err=_api.ref().executeQueue(queue);
         if (!err.isOk()) {
         fprintf(stderr, I18N("Error: %s\n"), err.errorString().c_str());
    }
    
    DBG_INFO(0, "Queue executed.");

    DBG_INFO(0, "Committing results");

    err=_api.ref().processOutboxResponse(queue.ref().response());
       if (!err.isOk()) {
              DBG_ERROR(0, "Error: %s", err.errorString().c_str());
              return HobitErrorExecution;
       }

    DBG_INFO(0, "Ok.");
    errorFlag=false;

    fprintf(stdout, I18N("Detailed job results:\n"));

	
     err1 = _AdminJobFactory->showJobResults();
     err2 = _AccountJobFactory->showJobResults();
	

    // Execution of JobQueue finished

       if (err1 != HobitErrorSuccess) 
         result = err1;
       else result = err2; 
     }
     catch (Error xerr) {
        emit logTextToShow(xerr.errorString().c_str());
        return HobitErrorExecution;
     }
    if (errorFlag)
       result = HobitErrorExecution;
    result = HobitErrorSuccess;
  }
  else result = HobitErrorNotConnected;
  if(result == HobitErrorSuccess)
    saveEnvironment();
  _Connector->closeConnection();
  return result;
}


/** base method for creating a new user */
HobitError HoBIT::createUser(const string &mediumname,
                             const string &mediumType,
                             bool readonly,
                             int country,
                             const string &instid,
                             const string &userid,
                             const string &customerid,
                             const string &server)
{
   deleteAllBanks();
                                                                                           
   Error err;
   err = _Database->createUser
     (mediumname,
      mediumType,
      readonly,
      country,
      instid,
      userid,
      customerid,
      server);
   
   if (!err.isOk() || !saveEnvironment ()){
       return HobitErrorSave;
       QMessageBox::critical(_parent, "Error", tr("Error while writing environment"));
   }
   return HobitErrorSuccess;
}



/** base method for creating a new user using a ddv card */
HobitError HoBIT::importUserDDV(bool readonly,
                                int country,
                                const string &instid,
                                const string &userid,
                                const string &customerid,
                                const string &server)
{
   deleteAllBanks();
                                                                                           
   Error err;
   err = _Database->importUserDDV(readonly, country,instid, userid, customerid, server);
   
   if (!err.isOk()){
       QMessageBox::critical(_parent, tr("Error"), tr("Error while attaching the Chipcard reader. \nPerhaps no reader is installed, the driver is missing \n or the LibChipCard2-Deamon has not been started yet"));
       return HobitErrorSave;
   }
   if (!saveEnvironment()){
       QMessageBox::critical(_parent, tr("Error"), tr("Error while writing environment"));
       return HobitErrorSave;
   }
   
   return HobitErrorSuccess;
}


/** Loads the OpenHBCI environment when the Program is started */
bool HoBIT::loadEnvironment(){
    int result = _Crypt->decryptConfigDataDir();
    if(result < -1){
      QMessageBox::critical(NULL, QObject::tr("Fatal Error"), QObject::tr("Failed to decrypt data. Aborting") );
      return false;
    }
    cerr << "Loading Environment....\n";
    string configdir = (_Crypt->getConfigDataDir() + "/").latin1();
    Error err = _api.ref().loadEnvironment( configdir + _openhbcifile, configdir + _aqmoneyfile);

    if (!err.isOk()) {
      DBG_ERROR(0, "Error loading environment (%s)",
                err.errorString().c_str());
                removeUncryptedEnvironment();
    return false;
   }
  
   err = _Database->readTransferTemplates(configdir + _templatefile);
   if (!err.isOk()) {
   DBG_ERROR(0, "Error loading templates (%s)",
             err.errorString().c_str());
             removeUncryptedEnvironment();
   return false;
  }

  removeUncryptedEnvironment();
  return true;
}


/** Saves the OpenHBCI environment and data  */
bool HoBIT::saveEnvironment(){
    
    // check if saving was possible
    cerr << "Writing environment.\n";
    string configdir = (_Crypt->getConfigDataDir() + "/").latin1();
    Error err = _api.ref().saveEnvironment(configdir + _openhbcifile, configdir + _aqmoneyfile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return false;
    }
  
    // check if saving was possible
    cerr << "Writing Templates.\n";
    err = _Database->saveTransferTemplates(configdir + _templatefile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving templates (%s)",
                err.errorString().c_str());
      return false;
    }
  
 /*  if(GWEN_Directory_GetPath(_hbcidatadir.c_str(),GWEN_PATH_FLAGS_PATHMUSTEXIST) != 0){ //ToDo: remove. This is done in GuiFindMedium 
       cerr << "creating moneypenny directory" << endl;
       string cmd = "mkdir " + _hbcidatadir;
       if(system(cmd.c_str()))
         perror("Failed to create moneypenny directory");
    }*/
    // now compress and crypt environment and datadir   
    int val = _Crypt->encryptConfigDataDir();
    
    if (val != 0) { 
   	showLogText(tr("Error when encrypting data."));
	return false;
     }
    
    removeUncryptedEnvironment();
    return true;
}



/** shows the logwindow and adds a logtext*/
void HoBIT::showLogText(QString string){
  emit logTextToShow(string);
}


/** creates a new security medium using the given information 
    * @ param mtype the medium type
    * @ param readonly decides whether the medium will be mounted readonly or not
    * @ param country the countrycode of the bank
    * @ param bankId the bankcode (bankleitzahl) of the institute
    * @ param userId the userId
    * @ param name the path of the new security file
    */
  HobitError HoBIT::createNewMedium  
   ( const string & mtype, 
     bool readonly, 
     int country, 
     const string & bankId, 
     const string & userId, 
     const string & keyname,
     const QString & newPassword)
{
    Pointer<Medium> md;
    try{	
      if(mtype == "RDHFile")
        _interactor.ref().setSecurityMode(HBCI_SECURITY_RDH);	
      else  		  
        _interactor.ref().setSecurityMode(HBCI_SECURITY_DDV);	
      _interactor.ref().setNewPassword(newPassword); //give it to the interactor which is called fram aqmoney
       md = _api.ref().createNewMedium(mtype, readonly,country, bankId, userId, keyname);
    }
    catch(...){
      qDebug("Error when trying to create a new medium.");
    }
    
    if (!md.isValid()) return HobitErrorExecution;
    else{ 
        return HobitErrorSuccess;
    }
	
}


  //-------------------------------------------------------------------------
  //  DB-Access Operations
  //-------------------------------------------------------------------------

  /** Returns a std::list which includes all registered users as a struct
    */
  list<Pointer<User> > HoBIT::listUsers(){
  return _api.ref().getUsers();
  }


  /** removes all existing banks from the gwen database */
  void HoBIT::deleteAllBanks(){
      _Database->deleteAllBanks();    
  }

  /** searches the api for a special bank */
  Pointer<HBCI::Bank> HoBIT::findBank(int country, const string& instId){
    return _api.ref().findBank(country, instId);
  }
  
  

  /** changes the used hbci version */
  bool HoBIT::changeHbciVersion(int val){
      bool ok = _Database->changeHbciVersion(val);
      if (ok)
       saveEnvironment();
      return ok;
  }
  
  
  /** creates a new account with the overgiven values
  * this procedure will for example be done, when 
  * the retrieving of the account list from the 
  * bank server fails 
  *
  * @param name: Name of the account owner
  * @param accountId: the account number
  * @param accountSubId: Hbci allows subids for accounts 
  * 			 this parameter is normaly not needed
  * @param accountName:  you can give your account an description
  *
  */		
HobitError HoBIT::addAccount(const string& name, const string &accountId, const string &accountSubId, const string& accountName){

 bool saved = false;

 showLogText(tr("Creating account"));
 HobitError err = _Database->addAccount(name, accountId, accountSubId, accountName);
 if (err != 0) 
    showLogText(tr("Your input was not correct - aborting."));
 else {
    showLogText(tr("Saving data"));
    saved = saveEnvironment();
    if (!saved) 
       showLogText(tr("An error occured while saving data."));
    else showLogText(tr("Data have been saved."));
 }
 
 showLogText("\n");  
 if (err == 0 && saved) 
    showLogText(tr("Your account has been created successfully."));
 else if (err != 0) 
        showLogText(tr("Error creating your account"));
 else if (!saved) 
        showLogText(tr("Error while saving the environment"));

 return err;
}
  
  
    /** changes the account description */
  void HoBIT::changeAccountName(const string& accid, const string& name){
  _Database->changeAccountName(accid, name);
  saveEnvironment();
  }

  
  /** get a list of all accounts */
  list<Pointer<Account> > HoBIT::listAccounts(){
     return _api.ref().getAccounts();
  }
  
  
  /** searches the aqmapi for a single account
   */
  Pointer<Account> HoBIT::findAccount(const string& accid)
  {
    Pointer<Account> no;
    list<Pointer<Account> > accounts;
    list<Pointer<Account> >::iterator accit;
       
    accounts =  _api.ref().getAccounts(0,
                                       "*",
                                       accid,
                                       "*");

    if (accounts.size()< 1){
            cerr << "No matching accounts found\n";
            return no;
    }
                                       
    if (accounts.size()>1){
            cerr << "Problem in data - to many possible accounts\n";
            return no;
    }
    
    accit = accounts.begin();
    return *accit;	
  }


   /** returns a list of transactios  */
  list<Pointer <Transaction> > HoBIT:: listTransactions
    (int country,
     const string &bankCode,
     const string &accountId,
     const string &accountSubId,
     const Date &fromDate,
     const Date &toDate)
 {

  return _Database->listTransactions
    (country,
     bankCode,
     accountId,
     accountSubId,
     fromDate,
     toDate);
 }

  
list <Pointer <DatedTransfer> >  HoBIT::listDatedTransfers
(int country,
 const string &bankCode,
 const string &accountId,
 const string &accountSubId)
{                                                                                    
    return _Database->listDatedTransfers(country, bankCode, accountId, accountSubId);
}


list <Pointer <StandingOrder> >  HoBIT::listStandingOrders
 ( int country,
   const string &bankCode,
   const string &accountId,
   const string &accountSubId)
{

    return _Database->listStandingOrders(country, bankCode, accountId, accountSubId);

}


/** generates the customers iniletter
*/
string HoBIT::generateIniLetter(){
     return _Database->generateIniLetter(0, "*", "*");
}


/** checks and returns the date of the last transaction stored */
QDate HoBIT::getLastDateOfTransactions(){
    Date d = _Database->getLastDateOfTransactions();

    if (!d.isValid()) return QDate(1,1,2000);

    QDate date(d.year(), d.month(), d.day());
    return date;
}


/**
  * returns a list of all known banks. Banks are 
  * returned as HoBIT_Bank-objects. 
  */
  list<HoBIT_Bank > HoBIT::listOfAllKnownBanks(){
      if( _banklist.empty()){  
         QString filename = MP_DIR()  + QString(MONEYPENNY_BANKDATA_DIR) + "/"+ QString(MONEYPENNY_ALL_BANKDATA_FILE);
         _banklist = _Database->readBankFile(filename.latin1()); 
      } 
      return _banklist;   
}

/**
  * returns a list of all known banks, that support HBCI. Banks are 
  * returned as HoBIT_Bank objects. 
  */
  list<HoBIT_Bank> HoBIT::listOfKnownHbciBanks(){
     if(_hbcibanklist.empty()){
       QString filename = MP_DIR()  + QString(MONEYPENNY_BANKDATA_DIR) + "/"+ QString(MONEYPENNY_HBCI_BANKDATA_FILE);
       _hbcibanklist = _Database->readBankFile(filename.latin1()); 
     }
     return _hbcibanklist;     
}


 /** returns the hash fingerprint of the institute after resceiving the bank servers public keys 
 */
  string HoBIT::getBankFingerPrint(int country, const string& bankcode, const string& customerId){
    return _Database->getBankFingerPrint();
  }

  
    /** returns the fingerprint of the current keypair*/
  string HoBIT::getMyOwnFingerPrint(int country, const string& bankcode, const string& customerId){
    return _Database->getMyOwnFingerPrint();
  }
  
  
  /** returns all transfer templates */
  list<Pointer<HoBIT_TransferTemplate> > HoBIT::getTransferTemplates(){
  return _Database->getTransferTemplates();
  }
  
  
  /** creates a new transfer template 
*/
void HoBIT::createTransferTemplate(	const string& ownAccountId,
				      	const string& otherName, 
			      		const string& otherAccountId,
			      		const string& otherBank,
			      		const Value& value,
			      		const string& purpose1,
			      		const string& purpose2)
{

Pointer<HoBIT_TransferTemplate> temp;
temp = new HoBIT_TransferTemplate (	ownAccountId,
					otherName, 
			      		otherAccountId,
			      		otherBank,
			      		value,
			      		purpose1,
			      		purpose2);

_Database->addTransferTemplate(temp);
saveEnvironment();
}


 /** creates a new transfer template 
*/
void HoBIT::deleteTransferTemplate(	const string& ownAccountId,
  			      		const string& otherName, 
			      		const string& otherAccountId,
			      		const string& otherBank,
			      		const string& value,
			      		const string& purpose1,
			      		const string& purpose2)
{
_Database->deleteTransferTemplate( ownAccountId,
					otherName, 
			      		otherAccountId,
			      		otherBank,
					value,
					purpose1,
			      		purpose2);
saveEnvironment();
}
  

//-------------------------------------------------------------------------
//  AdminJobs
//-------------------------------------------------------------------------

/** returns the account job queue */
  list<Pointer<AdminJob> > HoBIT::getAdminJobQueue(){
    return _AdminJobFactory->queue();
  }
  
  
  /** deleteAdminjob */
  void HoBIT::deleteAdminJob(int number){
    _AdminJobFactory->deleteJob(number);
  }

  
/** creates a job get keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetKeys(){
 HobitError err = _AdminJobFactory->createJobGetKeys();
 if (err == HobitErrorSuccess) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get keys, which will be executed directly */
HobitError HoBIT::createJobGetKeys(){
 return _AdminJobFactory->createJobGetKeys();
}


/** creates a job get systemid, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetSystemId(){
 HobitError err = _AdminJobFactory->createJobGetSystemId();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get systemid, which will be executed directly */
HobitError HoBIT::createJobGetSystemId(){
 return _AdminJobFactory->createJobGetSystemId();
}


/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetAccounts(){
 HobitError err = _AdminJobFactory->createJobGetAccounts();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get systemid, which will be executed directly */
HobitError HoBIT::createJobGetAccounts(){
 return _AdminJobFactory->createJobGetAccounts();
}



/** creates a job send keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobSendKeys(){
  HobitError err = _AdminJobFactory->createJobSendKeys();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobSendKeys(){
 return _AdminJobFactory->createJobSendKeys();
}


/** creates a job send keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetReports(){
 HobitError err = _AdminJobFactory->createJobGetReports();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobGetReports(){
 return _AdminJobFactory->createJobGetReports();
}



//-------------------------------------------------------------------------
//  Account Jobs
//-------------------------------------------------------------------------

/** returns the account job queue */
 list<Pointer<AccountJob> > HoBIT::getAccountJobQueue(){
  return _AccountJobFactory->queue();
 }


/** deleteAccountjob */
  void HoBIT::deleteAccountJob(int number){
   _AccountJobFactory->deleteJob(number);
  }

  
  
  
/** creates a job get balance, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetBalance(){
 HobitError err = _AccountJobFactory->createJobGetBalance();
 if (err == 0) _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobGetBalance(){
 return _AccountJobFactory->createJobGetBalance();
}


/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetTransactions(Date fdate, Date tdate, const string& accountId){
  HobitError err = _AccountJobFactory->createJobGetTransactions(fdate, tdate, 0, "*", accountId);
 cerr << err;
 if (err == 0) 
    _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobGetTransactions(Date fdate, Date tdate, const string& accountId){
  return _AccountJobFactory->createJobGetTransactions(fdate, tdate, 0, "*", accountId);
}



/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetDatedTransfers(Date fdate, Date tdate, const string& accountId){
  HobitError err = _AccountJobFactory->createJobGetDatedTransfers(0, "*", accountId, fdate, tdate);
 cerr << err;
 if (err == 0) 
    _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobGetDatedTransfers(Date fdate, Date tdate, const string& accountId){
  return _AccountJobFactory->createJobGetDatedTransfers(0, "*", accountId, fdate, tdate);
}



/** creates a job get standing orders, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetStandingOrders(const string& accountId){
 HobitError err = _AccountJobFactory->createJobGetStandingOrders(0, "*", accountId);
 cerr << err;
 if (err == 0) 
   _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get standing orders, which will be executed directly */
HobitError HoBIT::createJobGetStandingOrders(const string& accountId){
  return _AccountJobFactory->createJobGetStandingOrders(0, "*", accountId);
}



/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobSingleTransfer(bool skipBad,
                                                                            int country,
                                                                            const string &bankCode,
                                                                            const string &accountId,
                                                                            const string &accountSubId,
                                                                            int otherCountry,
                                                                            const string& otherBankCode,
                                                                            const string& otherAccount,
                                                                            const list<string> &otherName,
                                                                            const list<string> &purpose,
                                                                            const  Value&  value,
                                                                            int textkey,
                                                                            Date date){
                                                                              
    HobitError err = _AccountJobFactory->createJobSingleTransfer(skipBad,
                                                                                                   country,
                                                                                                   bankCode,
                                                                                                   accountId,
                                                                                                   accountSubId,
                                                                                                   otherCountry,
                                                                                                   otherBankCode,
                                                                                                   otherAccount,
                                                                                                   otherName,
                                                                                                    purpose,
                                                                                                    value,
                                                                                                    textkey,
                                                                                                    date);
   if (err == 0) _AccountJobFactory->executeJobQueue();
         return err;
}

*/
/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobSingleTransfer
 (bool skipBad,
  int country,
  const string &bankCode,
  const string &accountId,
  const string &accountSubId,
  int otherCountry,
  const string& otherBankCode,
  const string& otherAccount,
  const list<string> &otherName,
  const list<string> &purpose,
  const  Value&  value,
  int textkey,
  Date date)
{
   return _AccountJobFactory->createJobSingleTransfer
    (skipBad,
     country,
     bankCode,
     accountId,
     accountSubId,
     otherCountry,
     otherBankCode,
     otherAccount,
     otherName,
     purpose,
     value,
     textkey,
     date);
}



/** Deletes a  dated transfer */
HobitError HoBIT::createJobDeleteDatedTransfer(Pointer<DatedTransfer> dt){

list<string> purpose;
purpose.push_back("Purpose");

list<string> othername;
othername.push_back(dt.ref().otherName());

QString my = dt.ref().accountId().c_str();
QString other = dt.ref().otherAccountId().c_str();
QString id = dt.ref().jobId().c_str();

 return      _AccountJobFactory->createJobDeleteDatedTransfer
   ( false,                                     // skipbad
     dt.ref().country(),                        // country
     dt.ref().bankCode(),                   // bankcode
     (my.setNum(my.toInt())).ascii(),    // accountid
     "",                                           // accountSubId
     dt.ref().otherCountry(),           // other country
     dt.ref().otherBankId(),          // other bankcode
     (other.setNum(other.toInt())).ascii(),      // other account
     dt.ref().othernames(),           // other name
     dt.ref().purposes(),              // purpose
     dt.ref().value(),                  // value
     dt.ref().textKey(),               // textkey
     dt.ref().textKeyExt(),               // textkey
     dt.ref().date(),                   //date
     dt.ref().jobId());                  // jobid
}


    /** creates a new standing Order */
/*  HobitError HoBIT::createAndSendJobStartStandingOrder(bool skipBad,
                                                                                      int country,
                                                                                      const string &bankCode,
                                                                                      const string &accountId,
                                                                                      const string &accountSubId,
                                                                                      int otherCountry,
                                                                                      const string& otherBankCode,
                                                                                      const string& otherAccount,
                                                                                      const list<string> &otherName,
                                                                                      const list<string> &purpose,
                                                                                      const  Value&  value,
                                                                                      int textkey,
                                                                                     const Date& date,
                                                                                     const Date& firstdate,
                                                                                     const Date& lastdate,
                                                                                     const int turnus,
                                                                                     bool weekly,
                                                                                     int day){

   HobitError err =  _AccountJobFactory->createJobStartStandingOrder(skipBad,
                                                                                                           country,
                                                                                                            bankCode,
                                                                                                            accountId,
                                                                                                            accountSubId,
                                                                                                            otherCountry,
                                                                                                            otherBankCode,
                                                                                                            otherAccount,
                                                                                                            otherName,
                                                                                                            purpose,
                                                                                                            value,
                                                                                                            textkey,
                                                                                                            date,
                                                                                                            firstdate,
                                                                                                            lastdate,
                                                                                                            turnus,
                                                                                                            weekly,
                                                                                                            day);

   if (err == 0) err = _AccountJobFactory->executeJobQueue();
   return err;
                                                                                                                                                                                                    
  }

*/

  /** Deletes a  dated transfer */
HobitError HoBIT::createJobDeleteStandingOrder(Pointer<StandingOrder> sto){

QString my = sto.ref().accountId().c_str();
QString other = sto.ref().otherAccountId().c_str();


showLogText(sto.ref().jobId().c_str());

return _AccountJobFactory->createJobDeleteStandingOrder
  ( false,
    sto.ref().country(),                            // country
    sto.ref().bankCode(),                      // bankcode
    (my.setNum(my.toInt())).ascii(),        // accountid
    sto.ref().otherCountry(),                    // other country
    sto.ref().otherBankId(),                     // other bankcode
    (other.setNum(other.toInt())).ascii(),    // other account
    sto.ref().othernames(),                      // other name
    sto.ref().purposes(),                          // purpose
    sto.ref().value(),                               // value
    sto.ref().textKey(),                            // textkey
    sto.ref().textKeyExt(),                        // textkey
    sto.ref().date(),                                 //date
    sto.ref().firstDate(),                            // first date
    sto.ref().lastDate(),                            // last date
    sto.ref().jobId(),                                // jobId
    sto.ref().turnus(),                              // turnus
    sto.ref().day(),                                 // day
    sto.ref().weekly());                           // weekly

}

  

  

        /** creates a new standing Order */
  HobitError HoBIT::createJobStartStandingOrder(bool skipBad,
                                                int country,
                                                const string &bankCode,
                                                const string &accountId,
                                                const string &accountSubId,
                                                int otherCountry,
                                                const string& otherBankCode,
                                                const string& otherAccount,
                                                const list<string> &otherName,
                                                const list<string> &purpose,
                                                const  Value&  value,
                                                int textkey,
                                                const Date& date,
                                                const Date& firstdate,
                                                const Date& lastdate,
                                                const int turnus,
                                                bool weekly,
                                                int day){

   return _AccountJobFactory->createJobStartStandingOrder
   (skipBad,
    country,
    bankCode,
    accountId,
    accountSubId,
    otherCountry,
    otherBankCode,
    otherAccount,
    otherName,
    purpose,
    value,
    textkey,
    date,
    firstdate,
    lastdate,
    turnus,
    weekly,
    day);   
}


/*!
    \fn HoBIT::removeUncryptedEnvironment()
 */
/**
 * 
 */
void HoBIT::removeUncryptedEnvironment()
{
   QString rm = "rm " + _Crypt->getConfigDataDir() + "/" + _openhbcifile.c_str();
  // system(rm.latin1());
   
   rm = "rm " + _Crypt->getConfigDataDir() + "/" + _aqmoneyfile.c_str();
   // system(rm.latin1());
   
   rm = "rm " + _Crypt->getConfigDataDir() + "/" + _templatefile.c_str();
   // system(rm.latin1());
   
}


void HoBIT::removeConfigDataDir(){
  QString  rm = "rm -r " + _Crypt->getConfigDataDir();
   // system(rm.latin1());
}


/** Returns the name of the bank for the given id. Returns an empty string, if no bank could be found.
*/
QString HoBIT::getBankNameForId(const QString & id){
  bool bankFound = false;
  QString result = "";
  list<HoBIT_Bank> banks = listOfAllKnownBanks();
  list<HoBIT_Bank>::iterator bit;
  for (bit = banks.begin(); bit != banks.end() && !bankFound; bit++)
  {
    QString bankId = (*bit).getBankId();

    if (bankId == id)
    {
      result = (*bit).getBankName();
      bankFound = true;
    }
  }//end for
  return result;
 }
 
/** Returns the name (url) of the bank server
*/
QString HoBIT::getBankServer(int country, const string &bankId){
      Pointer<Bank> bank = _api.ref().findBank(country,bankId);
      string url = bank.ref().address();
      return QString(url.c_str());
}

/** deletes the aqmoney engine
*/
//void HoBIT::deleteApi(){//this will crash!!
  /*  _api.release();
    _database.release();
    _accountjobfactory.release();
    _adminjobfactory.release();
    _validator.release();*/
//}

/**Creates the aqmoney engine
*/ 
void HoBIT::createApi(){          
     // setting up version
    _api = new AqMoneyAPI(_readonly);
    _api.ref().setSystemName(_applicationname);
    _api.ref().setSystemVersion(VERSION);
    _api.ref().setApplicationName(_applicationname + " " + VERSION);
    
  // creating and setting interactor (take default authorization)
  // _interactor = new HoBIT_Interactor(_api.ptr(), _parent);
  // _api = new AqMoneyAPI(_readonly);
  // _api.ref().setSystemName(_applicationname);
  // _api.ref().setSystemVersion(VERSION);
  // _api.ref().setApplicationName(_applicationname + " " + VERSION);
    
  // creating and setting interactor and authentificator
    _interactor = new HoBIT_Interactor(_api.ptr(), _parent);
    _auth = new HoBIT_Auth(_api.ptr(), _parent);
    _api.ref().setInteractor(_interactor.cast<Interactor>());
    _api.ref().setAuthentificator(_auth.cast<Auth>());
 
   // creating and setting members
   _Database = new DatabaseAccess(_api);
    
   // now setting directories and file names
   //top directory is crypt::_ConfigDataDir 
   _api.ref().setAapplicationDataDir((_Crypt->getConfigDataDir() + "/").latin1() + _hbcidatadir); //take care of Aa !!
  
   // creating JobFactories
   _AccountJobFactory = new AccountJobFactory(_api.ptr());
   _AdminJobFactory = new AdminJobFactory(_api.ptr());
              
   // create validator 
   _validator = new HoBIT_Validator(_api.ptr());
}


/** Returns the previously choosen customer */
Pointer<Bank> HoBIT::getBank(){

    list<Pointer<Bank> > banks;
    Pointer<Bank> bank;
           
    banks =  _api.ref().banks();
    
    if (banks.size()< 1){
            cerr << "No matching bank found\n";
            return bank;
    }
                                       
    if (banks.size()>1){
            cerr << "Problem in data - to many possible banks\n";
            return bank;
    }
    
    bank = *banks.begin();
    return bank;	
}
/** Import an RDH2 formatted key. Format is specified in "FinTS-Format der RDH2 Diskette" by PPI, Hamburg. see www.hbc-kerneli.org
*/
bool HoBIT::importRDH2Key(const QString & fileName, const QString & password){
  //Under construction!!!
  qDebug("\nFunction HoBIT::importRDH2Key");
 // KeyFileRDH2 * keyFileRDH2 = new KeyFileRDH2(fileName, password);
  
  KeyFileOpenHbci * keyFileOpenHbci = new KeyFileOpenHbci(fileName, password);
  if(!keyFileOpenHbci->exists())
    return false;
  keyFileOpenHbci->readFile();
  qDebug("\nFunction HoBIT::importRDH2Key: keyFile read");
  
  delete keyFileOpenHbci;
  qDebug("HoBIT::importRDH2Key");
  return true;  
    
}

/** Returns true for acceptable and intermediate accountId strings
*/
bool HoBIT::validateAccountIdString(const QString & accountId){
  // regular expression allows only digits from 0 to 9 with a length of 0 to 15
 QRegExp rx( "[0-9]{0,15}" );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 QString localAccountId = accountId;
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 return (v.validate(localAccountId, pos)==QValidator::Acceptable)  || (v.validate(localAccountId, pos)==QValidator::Intermediate);
}

/** Read property of AdminJobFactory m_AdminJobFactory. 
*/
AdminJobFactory * HoBIT::getAdminJobFactory()const{
	return _AdminJobFactory;
}
/** Read property of AccountJobFactory m_AccountJobFactory. 
*/
AccountJobFactory * HoBIT::getAccountJobFactory()const{
	return _AccountJobFactory;
}



/** returns the current HBCI mode */
HBCI_MODE HoBIT::getHbciMode(){
     list<Pointer<Customer> > custs=_api.ref().getCustomers();
     list<Pointer<Customer> >::iterator cit;
     Pointer<Medium> medium;
    
      if (custs.size() != 1) {
            cerr << "No or to many customers found. Number of customers: " << custs.size()<<"\n";
	    return HBCI_MODE_ERROR;
        }
      else {
        cit=custs.begin();
        medium=(*cit).ref().user().ref().medium();
	}

	if (medium.ref().securityMode() == HBCI_SECURITY_RDH) 
         return HBCI_MODE_RDH;
	else if (medium.ref().securityMode() == HBCI_SECURITY_DDV) 
           return HBCI_MODE_DDV;
	else return HBCI_MODE_ERROR;
}


/** Returns true, if no jobs are in job queues
*/
bool HoBIT::areJobQueuesEmpty(){
  list<Pointer<AccountJob> > accountjoblist = getAccountJobQueue();
  list<Pointer<AdminJob> > adminjoblist = getAdminJobQueue();

  return (accountjoblist.empty() && adminjoblist.empty());
}

/** Returns the user from OpenHbci database. Return value should be checked with isValid().
*/
Pointer<User> HoBIT::getUser(){
  Pointer<Bank> bank = getBank();;
  Pointer<User> user; //constructor creates an invalid Pointer
  list<Pointer<User> > users;
   
  if (bank.isValid()){  
    users = bank.ref().users();    
    if(users.size() > 0){
      user = *users.begin();
    }
  }
  return user;
}

/** Returns the currently used medium from OpenHbci database. Return value should be checked with isValid().
*/
Pointer<Medium> HoBIT::getMedium(){
   Pointer<Medium> medium;
   Pointer<User> user = getUser();
   if(user.isValid())
     medium = user.ref().medium();
   return medium; 
}

/** Returns the first customer from the user's customer list. Return value should be checked with isValid().
*/
Pointer<Customer> HoBIT::getCustomer(){
   Pointer<User> user = getUser();
   list<Pointer<Customer> > customers;
   Pointer<Customer> customer;
   if(user.isValid()){
     customers = user.ref().customers();     
     if (customers.size() > 0)        
       customer = *customers.begin();
   } 
   return customer; 
 }
 
/** Returns true, if cardreader is up.
*/
bool HoBIT::isCardReaderOk(){
 bool result;
 
 //check for an installed plugin
 string cardName = ""; //let the plugin create the name
 Pointer<MediumPluginList> pluginList =_api.ref().enumerateMediumPlugins();
 list< Pointer< MediumPlugin > > plugins = pluginList.ref().plugins();
 list< Pointer< MediumPlugin > >::iterator it; //this is the current plugin
 if(plugins.empty())
   result = false;
 else{
   for(it = plugins.begin(); it != plugins.end(); it++){// check for a plugin that supports cards
    if((*it).ref().mediumType() == MediumTypeCard){
      if((*it).ref().mediumCheck(cardName).isOk()) //checks for an accessible card
       result = true;
    }  
   }//end for   
 }//end else
 if(!result){
     QMessageBox::critical(_parent, tr("Error"), 
    tr("Can not load ddvcard2 plugin. Check your libchipcard installation."));
    return result;
 }
 
//check for a running chipcardd2 demon
 int returnValue = system("ps -A | grep chipcardd2"); //check for a running chipcarddd2 process
 if(returnValue != 0){
   bool okPressed;
   QString rootPassword = QInputDialog::getText( tr("Information"),
       tr("Can not read chipcard. Driver daemon chipcardd2 is not yet running. \nEnter root password to let MoneyPenny try to start it."),
        QLineEdit::Password,
        0,
        &okPressed);
   if(okPressed){ //try to start daemon
    QString sudoCommand = "echo " + rootPassword + " | sudo -S /usr/sbin/chipcardd2 --pidfile /var/run/chipcardd2.pid --logfile /var/log/chipcardd2.log";
    system(sudoCommand.latin1());  
    system("sudo -K"); //kill the sudo timestamp         
    QMessageBox::information(_parent, tr("Information"), //this gives the daemon time to start, not very smart, but it works
    tr("MoneyPenny has started daemon with command: \n/usr/sbin/chipcardd2 --pidfile /var/run/chipcardd2.pid"));
   }  
   
   int returnValue = system("ps -A | grep chipcardd2"); //check again for a running chipcarddd2 process
   if(returnValue != 0){ //still not ok, give up
    QMessageBox::critical(_parent, tr("Installation Error"), 
    tr("Starting daemon chipcardd2 failed. \nCheck your libchipcard2 installation"));
    result = false;
   }
 }//end if returnValue  
 
 return result;
}
