/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hobit_crypt.h"

#include <qstring.h>
#include <qdatetime.h>
#include <qprogressdialog.h>
#include <iostream>
#include <fstream>
#include <qdir.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>


//#define DEBUG_CRYPT 


using namespace std;

HoBIT_Crypt::HoBIT_Crypt()
{
 	_CryptDir = "/home"; //must be set later
	_ConfigDataDir = "/home";  //must be set later
        
        _ChecksumDigestType = EVP_ripemd160();
        _KeyDigestType = EVP_ripemd160();
	_MsgDigestContext = EVP_MD_CTX_create();

	_CipherType = EVP_bf_cbc();
        EVP_add_cipher(_CipherType);
	_ModuleId = EVP_CIPHER_nid(_CipherType);
	_IniVectorLength = EVP_CIPHER_iv_length(_CipherType);
	_BlockSize = EVP_CIPHER_block_size(_CipherType);

        //create a new initialisation vector for this session
	_NewIniVector.resize(_IniVectorLength);
	srand((unsigned)time(NULL));
        for(uint ivpos = 0; ivpos < _NewIniVector.strLength(); ivpos++)
    	   _NewIniVector[ivpos] = (unsigned) rand();
	_OldIniVector.resize(_IniVectorLength);
		
#ifdef DEBUG_CRYPT
	cout << "Data dir: " << _ConfigDataDir << endl;
	cout << "NID: " << _ModuleId << endl;
        cout <<"EVP_MAX_MD_SIZE : " << EVP_MAX_MD_SIZE << endl;
	cout << "Ini vector length: " << _IniVectorLength << endl;
        QString sIni, sNum;
        for (int pos = 0; pos < _IniVectorLength; pos++)
         sIni += sNum.setNum(_NewIniVector[pos], 16); 
	cout << "New Ini vector: " << sIni.latin1() << endl;
	cout << "Blocksize: " << _BlockSize << endl;
	//cout << "Password: " << _Password << endl;
	cout << "OLDIV: Not yet valid" << endl;
	cout << "NEWIV: " << _NewIniVector.toHexString(0) << endl;
#endif
	
}


HoBIT_Crypt::~HoBIT_Crypt()
{
	//EVP_CIPHER_CTX_cleanup(&_CipherContext);
	EVP_MD_CTX_destroy(_MsgDigestContext);
        qDebug("Destructor HoBIT_Crypt finished");
}


int HoBIT_Crypt::encryptFile (FILE * infile, FILE * cryptfile)
{
   int result;
   // save the new init vector before using it
    if(saveInitVector() != 0){
	cerr << "Failed to save init vector" << endl;
	return -1;
     }

  int mode = 0;
    
  result= docrypt(infile, cryptfile, mode, _NewIniVector);
  return result;
}

int HoBIT_Crypt::decryptFile(FILE * infile, FILE * outfile)
{
   int result;
  //init old initialisation vector from file
       if(loadInitVector() != 0){
	 cerr << "Failed to load init vector" << endl;
	 return -1;
	}
	
  int mode = 1;
    
  result= docrypt(infile, outfile, mode, _OldIniVector);
  return result;
}

int HoBIT_Crypt::docrypt(FILE * infile, FILE * outfile, int mode, UCharArray & initVector)
{
   //allow enough space for input and output
   unsigned char inbuf[1024];
   unsigned char outbuf[1024 + EVP_MAX_BLOCK_LENGTH];
  // string testName = _ConfigDataDir + "/" + MONEYPENNY_DATADIR_NAME +"/testfile.tar.gz";
   int inlen, outlen;   
   int totalin = 0;
   int totalout = 0;
   /* Don't set _Password or IV because we will modify the parameters */
    EVP_CIPHER_CTX_init(&_CipherContext);
    EVP_CipherInit_ex(&_CipherContext, _CipherType, NULL, NULL, NULL, mode);
    EVP_CIPHER_CTX_set_key_length(&_CipherContext, _Key.strLength());
   /* We finished modifying parameters so now we can set key and initialization vector */
    EVP_CipherInit_ex(&_CipherContext, NULL, NULL, _Key.data(), initVector.data(), mode);
    int nLine = 1; 
     for(;;){
          inlen = fread(inbuf, 1, 1024, infile);
          totalin+= inlen;
          if(inlen <= 0) break;
          if(EVP_CipherUpdate(&_CipherContext, outbuf, &outlen, inbuf, inlen)){
            totalout += fwrite(outbuf, 1, outlen, outfile);
            nLine ++;
          }    
          else return -1; //error
       }//End for
      if(EVP_CipherFinal_ex(&_CipherContext, outbuf, &outlen))                     
        fwrite(outbuf, 1, outlen, outfile);
      else return -1; //error 
        cerr << "Crypt: Total inlen: " << totalin << " Total outlen: "<< totalout <<"\n";
      
   //   EVP_CIPHER_CTX_cleanup(&ctx);
      return 0;
}


int HoBIT_Crypt::encryptConfigDataDir()
{
     FILE * tarFile, * cryptFile;
     //compress specified files and write them to the pipe on stdout (--file=- is stdout)
     QString tarcmd = "tar --create --gzip --file=- " + _ConfigDataDir + "/" + MONEYPENNY_HBCIDATA_DIR + // file =- means: use stdout for output
     " " +  _ConfigDataDir + "/" + MONEYPENNY_AQMONEY_FILE +
     " " + _ConfigDataDir + "/" + MONEYPENNY_OPENHBCI_FILE +
     " " + _ConfigDataDir + "/" + MONEYPENNY_TEMPLATE_FILE;
     cerr << "tar command: " << tarcmd.latin1() << "\n";
             
    //create the pipe for tar's output and start tar
     tarFile = popen(tarcmd.latin1(),"r");
     if(!tarFile){
             cerr<<"encryptConfigDataDir: Failed to execute tar command";
             return -1;
     }
       
     //build outfile string
     QString cryptFileName = _CryptDir + "/" + MONEYPENNY_ENCRYPTED_CONF_FILE;
     cryptFile = fopen(cryptFileName.latin1(), "wb");
     if(!cryptFile){
       cerr<<"encryptConfigDataDir: Failed to open cryptFile for writing: " << cryptFileName << endl;
       return -2;
     }
     if(encryptFile(tarFile,cryptFile) < 0) //read from the pipe and write to cryptFile
       cerr<<"encryptConfigDataDir: Error when encrypting datadir";
     pclose(tarFile); 
     fclose(cryptFile);
     return 0;
}


int HoBIT_Crypt::decryptConfigDataDir()
{
    FILE * infile, * outfile;
    QString inFileName;
    inFileName = _CryptDir + "/" + MONEYPENNY_ENCRYPTED_CONF_FILE;
    //extract decrypted data read from the pipe on stdin and write them to datadir
    QString tarcmd = "tar --extract --gzip --file=- --directory='/'";// tar archive contains absolute path + _ConfigDataDir; 
    
    infile = fopen(inFileName.latin1(), "ro");   
    if(infile)
     outfile = popen(tarcmd.latin1(), "w"); //open the pipe for writing decrypted data to tar
    else{
     cerr << "decryptDatadir: Open infile failed: " << inFileName << endl;
     return -1;
    }
    if(!outfile){
     cerr << "decryptDatadir: Open pipe for output failed";
     fclose(infile);
     return -2;
    } 
    if(decryptFile(infile, outfile) < 0){
            cerr << "decryptDatadir: Failed to decrypt file: "<< inFileName << endl;
            fclose(infile);
            pclose(outfile);
            return -3;
    }
    fclose(infile);
    pclose(outfile); //close the pipe	
    return 0;
}




/*!
    \fn HoBIT_Crypt::loadInitVector
 */
int HoBIT_Crypt::loadInitVector()
{	
	QString ivpath = _CryptDir + "/iv.dat";
	ifstream ivin(ivpath.latin1(),ios_base::binary);
	if(!ivin){
		cerr << "Failed to open iv.dat file for data decryption" << endl;
		return -1;
	}
	char c;
	for(uint niv=0; niv < _OldIniVector.strLength(); niv++){
		ivin.get(c);
		_OldIniVector[niv] = (unsigned char) c;
	}
	ivin.close();
#ifdef DEBUG_CRYPT
	cout << "Old Ini vector loaded: " << _OldIniVector.toHexString(0) << endl;
#endif
	
	return 0;
}


/*!
    \fn HoBIT_Crypt::saveInitVector()
 */
int HoBIT_Crypt::saveInitVector()
{     
 //store new init-vector before using it
     QString ivpath = _CryptDir + "/iv.dat";
     
 #ifdef DEBUG_CRYPT 
     QFile ivFile (_CryptDir + "/iv.dat");
     if(ivFile.exists()){
             cerr << "deleting old iv.dat file" << ivpath << endl;
     } else {
             cerr << "iv.dat file does not exist! Creating new one" << endl;
     }
 #endif

     QString newivpath = _CryptDir + "/iv.dat";
     ofstream ivout(newivpath.latin1(),ios_base::binary);
     if(!ivout){
             cerr << "Failed to open iv.dat file for data decryption" << endl;
             return -1;
     }
     for(int niv=0; niv<_IniVectorLength; niv++)
             ivout.put((const char) _NewIniVector[niv]);
     ivout.close();

     return 0;
}


/*!
    \fn HoBIT_Crypt::savePasswordMD(const string &filepath)
 */
int HoBIT_Crypt::savePasswordMD(const QString &filepath)
{
   QString fp;
   if(filepath.isEmpty()){
           fp = _CryptDir + "/" + MONEYPENNY_PASSWORD_FILE;
   } else {
           fp = filepath;
   }
   cerr << "Password path:" << fp << endl;
   ofstream ofmd(fp.latin1(),ios_base::binary);
   if(!ofmd){
           cerr << "Failed to open file " << filepath << endl;
           return -1;
   }
   for(unsigned int i=0; i< _Checksum.strLength(); i++)
           ofmd.put(_Checksum[i]);
   ofmd.close();
   
   return 0;
}


/*!
    \fn HoBIT_Crypt::checkPasswordMD(const string &filepath = 0)
 */
int HoBIT_Crypt::checkPasswordMD(const QString &filepath)
{
   QString fileName;
   if(filepath.isEmpty())
        fileName = _CryptDir + "/" + MONEYPENNY_PASSWORD_FILE;
   else 
        fileName = filepath;
   
   ifstream stream(fileName.latin1(),ios_base::binary);
   if(!stream){
     cerr << "Failed to open password file " << fileName << endl;
     return -1;
   }
   UCharArray oldchecksum;
   oldchecksum.resize(EVP_MD_size(_ChecksumDigestType));
   char c;
   uint pos = 0;
   while(stream.get(c)&& (pos <= oldchecksum.strLength())){
         oldchecksum[pos] = (unsigned) c;
         pos++;
   }
   stream.close();
   if(oldchecksum == _Checksum) //password ok
      return 0;
   else return -2;   //wrong password
}


/*!
    \fn HoBIT_Crypt::setPassword(const string &password)
 */
bool HoBIT_Crypt::setPassword(const QString &password)
{
  UCharArray passwordArray; // keep it on the stack
  passwordArray.loadFromString(password);
  //derive key
  _Key.resize(EVP_MD_size(_KeyDigestType)); //make it big enough
  unsigned int keyLength;
  EVP_DigestInit(_MsgDigestContext, _KeyDigestType);
  EVP_DigestUpdate(_MsgDigestContext, passwordArray.data(), passwordArray.strLength());
  EVP_DigestFinal(_MsgDigestContext, _Key.data(),&keyLength);
  _Key.resize(keyLength); //resize to correct size, just for safety

  //Overwrite the password copy, just for paranoia
  for(uint i = 0; i < passwordArray.strLength(); i++)
    passwordArray[i] = (unsigned)rand();
  
  //derive Checksum
  _Checksum.resize(EVP_MD_size(_ChecksumDigestType));
  unsigned int checksumLength;
  EVP_DigestInit(_MsgDigestContext, _ChecksumDigestType);
  EVP_DigestUpdate(_MsgDigestContext, _Key, _Key.strLength());
  EVP_DigestFinal(_MsgDigestContext, _Checksum.data(), &checksumLength);
  
  return checksumLength == _Checksum.strLength(); //These must be identical, if everything was ok
}

/** Read property of string _ConfigDataDir. 
*/
const QString& HoBIT_Crypt::getConfigDataDir()const{
	return _ConfigDataDir;
}

/** Write property of string m_dataDir. 
*/
void HoBIT_Crypt::setConfigDataDir( const QString& _newVal){
	_ConfigDataDir = _newVal;
}
/** Write property of string _CryptDir. 
*/
void HoBIT_Crypt::setCryptDir( const QString& _newVal){
	_CryptDir = _newVal;
}

/** Read property of string _CryptDir. 
*/
const QString& HoBIT_Crypt::getCryptDir()const{
	return _CryptDir;
}
