/***************************************************************************
                          listviewitems.cpp  -  description
                             -------------------
    begin                : Mo Nov 15 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "listviewitems.h"
#include <openhbci2/date.h>
#include <qstring.h>
#include <qdatetime.h>
#include <string>
#include <qlabel.h>

using namespace std;

DatedTransferItem::DatedTransferItem(QTable* table, int row, Pointer<DatedTransfer> dt) {

 _DatedTransfer = dt;
 QString sValue = QString::number(dt.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"
 Date d = dt.ref().date();
 QDate date(d.year(), d.month(), d.day());
 if(table->numRows() < row +1)
   table->setNumRows(row +1);
   
 _Date = new QTableItem(table, QTableItem::Never, date.toString() );
 _Date->setWordWrap(true);
 table->setItem( row, 0, _Date);
 
 _Purpose = new QTableItem(table, QTableItem::Never, QString(dt.ref().purpose().c_str()) );
 _Purpose->setWordWrap(true);
 table->setItem( row, 1, _Purpose);
   
 _Value = new QTableItem(table, QTableItem::Never, sValue);
 table->setItem( row, 2, _Value);
 table->adjustRow(row);
}

DatedTransferItem::~DatedTransferItem(){
}


/** Encodes the Transaction data as an html- table row string.
  */
QString DatedTransferItem::htmlTableRow(const QString & bgColor){
  QString result;
  Date d = _DatedTransfer.ref().date();
  QDate date(d.year(), d.month(), d.day());
  string value = QString::number(_DatedTransfer.ref().value().getValue(),'f',2).latin1(); // 2 digits after . "-999.99"  
  string ouraccId = _DatedTransfer.ref().accountId();
  string ourbankCode = _DatedTransfer.ref().bankCode();
  string otheraccId = _DatedTransfer.ref().otherAccountId();
  string otherbankCode = _DatedTransfer.ref().otherBankId();
  string othername = _DatedTransfer.ref().otherName();
  string purpose = _DatedTransfer.ref().purpose();

  
 result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString();
   result += "</td>";
   
   result += "<td>";
    result += QString(othername.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(purpose.c_str());
   result += "</td>";

   result += "<td>";
    result += QString(otherbankCode.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(otheraccId.c_str());
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += QString(value.c_str());
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}


/** Read property of DatedTransfer _DatedTransfer. */
const Pointer<DatedTransfer> DatedTransferItem::datedTransfer(){
	return _DatedTransfer;
}


//#####################################################################################################################



TransactionItem::TransactionItem(QTable * table, int row, Pointer<Transaction> t){
  
   _Transaction = t;
  
  QTableItem *trDate;
  QTableItem *otherName;
  //QTableItem *custRef;
  QTableItem *bankCode;
  QTableItem *accountId;
  //QTableItem *textKey;
  //QTableItem *key;
  QTableItem *purpose;
  QTableItem *trText;
  QLabel *  labelValue;  //label allows to set the text color

  QString sValue = QString::number(t.ref().value().getValue(),'f',2) + "  "; // 2 digits after . "-999.99"
  Date d = t.ref().date();
  QDate date(d.year(), d.month(), d.day());
  if(table->numRows() < row +1)
   table->setNumRows(row +1);
   
  trDate = new QTableItem(table, QTableItem::Never, date.toString(QObject::tr("ddd,MMM,d yyyy", "Format string for the date")) );
  trDate->setWordWrap(false);
  table->setItem( row, 0, trDate);
  
  otherName = new QTableItem(table, QTableItem::Never, QString(t.ref().othername().c_str()) );
  otherName->setWordWrap(true);
  table->setItem( row, 1, otherName);
   
  labelValue = new QLabel(sValue, table);
  labelValue->setAlignment(Qt::AlignRight); //set alignment here, but set color in paintEvent!
  table->setCellWidget( row, 2, labelValue);
  
  list<string>::iterator line;
  list<string>lines = t.ref().purposeLines();
  QString purposeText;
  for(line = lines.begin(); line!= lines.end(); line++) {//chain the lines together
   if(!(*line).empty())
     purposeText += QString((*line).c_str()) + "\n";
  } 
  purposeText.truncate(purposeText.length() -1); //delete last \n
  purpose = new QTableItem(table, QTableItem::Never, purposeText );
  purpose->setWordWrap(true);
  table->setItem( row, 3, purpose);
 
  bankCode = new QTableItem(table, QTableItem::Never, QString(t.ref().otherbankcode().c_str()) );
  bankCode->setWordWrap(true);
  table->setItem( row, 4, bankCode);
  
  accountId = new QTableItem(table, QTableItem::Never, QString(t.ref().otheraccountId().c_str()));
  accountId->setWordWrap(true);
  table->setItem( row, 5, accountId);
  
  trText = new QTableItem(table, QTableItem::Never, QString(t.ref().text().c_str()) );
  trText->setWordWrap(true);
  table->setItem( row, 6, trText);

 /* _CustRef = new QTableItem(table, QTableItem::Never, t.ref().custref().c_str() );
  _CustRef->setWordWrap(true);
  table->setItem( row, 1, _CustRef);
 */ 
  table->adjustRow(row);
    
}

TransactionItem::~TransactionItem(){
 //do not delete table items, they are owned by table
}

/** Read property of DatedTransfer _datedTransfer. */
const Pointer<Transaction> TransactionItem::transaction(){
	return _Transaction;
}


/** Encodes the Transaction data as an html- table row string.
*/
QString TransactionItem::htmlTableRow(const QString & bgColor){
  QString result;
  Date d = _Transaction.ref().date();
  QDate date(d.year(), d.month(), d.day());
  QString value = QString::number(_Transaction.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"  
  QString ouraccId = _Transaction.ref().ouraccountId().c_str();
  QString ourbankCode = _Transaction.ref().ourbankcode().c_str();
  QString otheraccId = _Transaction.ref().otheraccountId().c_str();
  QString otherbankCode = _Transaction.ref().otherbankcode().c_str();
  QString othername = _Transaction.ref().othername().c_str();
  QString custref = _Transaction.ref().custref().c_str();
  QString key = _Transaction.ref().key().c_str();
  QString primanota = _Transaction.ref().primanota().c_str();
  
  list<string>::iterator line;
  list<string>lines = _Transaction.ref().purposeLines();
  QString purposeText;
  for(line = lines.begin(); line!= lines.end(); line++) {//chain the lines together
   if(!(*line).empty())
     purposeText += QString((*line).c_str()) + "<br>";
  } 
  purposeText.truncate(purposeText.length() -4); //delete last <br>
  
  string text = _Transaction.ref().text();
  
   result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString();
   result += "</td>";
   
   result += "<td>";
    result += othername;
   result += "</td>";
   
   result += "<td>";
    result += purposeText;
   result += "</td>";
   
   result += "<td>";
    result += text;
   result += "</td>";
   
   result += "<td>";
    result += otherbankCode;
   result += "</td>";
   
   result += "<td>";
    result += otheraccId;
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += value;
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}


//#####################################################################################################################


StandingOrderItem::StandingOrderItem(QTable * table, int row, Pointer<StandingOrder> so){

  _StandingOrder = so;

  string otherName = so.ref().otherName();
  string purp = so.ref().purpose();
  QString sValue = QString::number(so.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"
  if(table->numRows() < row +1)
   table->setNumRows(row +1);
  
  
  _OtherName = new QTableItem(table, QTableItem::Never, QString(so.ref().otherName().c_str()) );
  _OtherName->setWordWrap(true);
  table->setItem( row, 0, _OtherName);
   
  _Purpose = new QTableItem(table, QTableItem::Never, QString(so.ref().purpose().c_str()) );
  _Purpose->setWordWrap(true);
  table->setItem( row, 1, _Purpose);
  
  _Value = new QTableItem(table, QTableItem::Never, sValue);
  _Purpose->setWordWrap(true);
  table->setItem( row, 2, _Value);
}

StandingOrderItem::~StandingOrderItem(){
}

/** Read property of DatedTransfer _datedTransfer. */
const Pointer<StandingOrder> StandingOrderItem::standingOrder(){
	return _StandingOrder;
}



 /** Encodes the Transaction data as an html- table row string.
  */
QString StandingOrderItem::htmlTableRow(const QString & bgColor){
  
  QString result;
  Date d = _StandingOrder.ref().date();
  QDate date(d.year(), d.month(), d.day());
  d = _StandingOrder.ref().firstDate();
  QDate firstdate(d.year(), d.month(), d.day());
  d = _StandingOrder.ref().lastDate();
  QDate lastdate(d.year(), d.month(), d.day());
  QString day(QString::number(_StandingOrder.ref().day()));
  QString turnus(QString::number(_StandingOrder.ref().turnus()));
  string unit = _StandingOrder.ref().unit();
  QString weekly(QString::number(_StandingOrder.ref().weekly()));
  string yearly = _StandingOrder.ref().yearly();
  
  string value = QString::number(_StandingOrder.ref().value().getValue(),'f',2).latin1(); // 2 digits after . "-999.99"  
  string ouraccId = _StandingOrder.ref().accountId();
  string ourbankCode = _StandingOrder.ref().bankCode();
  string otheraccId = _StandingOrder.ref().otherAccountId();
  string otherbankCode = _StandingOrder.ref().otherBankId();
  string othername = _StandingOrder.ref().otherName();
  string purpose = _StandingOrder.ref().purpose();

  
 result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString();
   result += "</td>";
   
   result += "<td>";
    result += QString(othername.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(purpose.c_str());
   result += "</td>";

   result += "<td>";
    result += QString(otherbankCode.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(otheraccId.c_str());
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += QString(value.c_str());
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}

